<?php
/**
* Hot Property
*
* This extension will give the SEF Advance style URLs to Hot Property
* Place this file (sef_ext.php) in the main component directory:
*		/components/com_hotproperty/
*
* For SEF Advance > v3.7
*
* @package Hot Property 1.0.1
* @copyright (C) 2004-2006 Lee Cher Yeong, 2010 Emir Sakic
* @url http://www.sakic.net/
**/

define('_HP_SEF_TYPES', 'types');
define('_HP_SEF_PROPERTIES', 'properties');
define('_HP_SEF_COMPANIES', 'companies');
define('_HP_SEF_CONTACT', 'contact');
define('_HP_SEF_AGENTS', 'agents');
define('_HP_SEF_FEATURED', 'featured');
define('_HP_SEF_MANAGE', 'manage');
define('_HP_SEF_SEARCHES', 'searches');
define('_HP_SEF_RESULTS', 'results');

class sef_hotproperty {

	/********************************************************
	* CREATE
	********************************************************/

	/**
	* Creates the SEF advance URL out of the Mambo request
	* Input: $string, string, The request URL (index.php?option=com_example&Itemid=$Itemid)
	* Output: $sefstring, string, SEF advance URL ($var1/$var2/)
	**/
	function create ($string) {
	
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}
		
		//$string = preg_replace('/\[[0-9]+\]/','',$string);
		
		$sefstring = '';

		# View Property
		// index.php?option=com_hotproperty&view=properties&layout=property&id=56&Itemid=41
		if (stristr($string,'&amp;view=properties')) {

			if (stristr($string,'&amp;id=')) {
				$temp = explode("&amp;id=", $string);
				$temp = explode("&", $temp[1]);
				$property_id = $temp[0];
			}

			if ( isset($property_id) && is_numeric($property_id) ) {
				$db->setQuery( "SELECT t.name AS type_name, p.name AS property_name FROM #__hp_properties AS p" 
					.	"\nLEFT JOIN #__hp_prop_types AS t ON p.type = t.id"
					.	"\nWHERE p.id = '".$property_id."'"
					.	"\nLIMIT 1"
				);
				$rows = $db->loadObjectList();
				$row = $rows[0];

				$sefstring .= sefencode($row->type_name).'/'.sefencode($row->property_name).'/';
			} else if (stristr($string,'&amp;layout=manage')) {
				# Manage Property
					$sefstring .= _HP_SEF_MANAGE.'/';
					
					// ordering and limits
					$sefstring .= $this->append_ordering_and_limits($string);
			} else {
				$sefstring .= _HP_SEF_PROPERTIES."/";
				
				// ordering and limits
				$sefstring .= $this->append_ordering_and_limits($string);
				
			}

		}

		# View Type
		// index.php?option=com_hotproperty&view=types&layout=properties&id=6&Itemid=41
		if (stristr($string,'&amp;view=types')) {

			if (stristr($string,'&amp;id=')) {
			
				$type_id = $this->extract('id',$string);
				
				if ( $type_id && is_numeric($type_id) ) {
					$db->setQuery( "SELECT t.name AS type_name FROM #__hp_prop_types AS t" 
						.	"\nWHERE t.id = '".$type_id."'"
						.	"\nLIMIT 1"
					);
					$rows = $db->loadObjectList();
					$row = $rows[0];

					$sefstring .= sefencode($row->type_name).'/';
				}
			} else {
				$sefstring .= _HP_SEF_TYPES.'/';
			}
			
			// ordering and limits
			$sefstring .= $this->append_ordering_and_limits($string);

		}

		# View Featured
		if (stristr($string,'&amp;view=featured')) {

			$sefstring .= _HP_SEF_FEATURED.'/';

			// ordering and limits
			$sefstring .= $this->append_ordering_and_limits($string);

		}

		# View Agent
		if ( stristr($string,'&amp;view=agents') ) {
		
			// index.php?option=com_hotproperty&view=agents&layout=properties&id=3&Itemid=41
			
			$agent_id = $this->extract('id',$string);

			// This condition allows listagent module to work so that $agent_id can be a javascript expression.
			if ( is_numeric($agent_id) ) {
				$db->setQuery( "SELECT a.name AS agent_name, c.name AS company_name FROM #__hp_agents AS a" 
					.	"\nLEFT JOIN #__hp_companies AS c ON a.company = c.id"
					.	"\nWHERE a.id = '".$agent_id."'"
					.	"\nLIMIT 1"
				);
				$rows = $db->loadObjectList();
				$row = $rows[0];
			}
			
			// Default virtual directory would be: _HP_SEF_COMPANIES/company_name/agent_name/.
			// There is an alternative way to represent this as to allow List Agents module to work: _HP_SEF_AGENTS/agent_id
			if ( !is_numeric($agent_id) || count($row) <= 0 ) {
				$sefstring .= _HP_SEF_AGENTS.'/'.$agent_id;
			} else {
				$sefstring .= _HP_SEF_COMPANIES.'/'.sefencode($row->company_name).'/'.sefencode($row->agent_name).'/';
			}
			
			# View Agent's Properties
			if ( stristr($string,"&amp;layout=properties") ) {

				// ordering and limits
				$sefstring .= $this->append_ordering_and_limits($string);
				
			# Send Enquiry Email
			} else if ( stristr($string,'&amp;layout=contact') ) {
				$sefstring .= _HP_SEF_CONTACT.'/';
				$referer = $this->extract('referer',$string);
				if ($referer) {
					$sefstring .= $this->append_query_string($sefstring,'referer',$referer);
				}
			} else {
				// ordering and limits
				$sefstring .= $this->append_ordering_and_limits($string);
			}

		}

		# View Company / View Company Email / Send Company Email
		if ( stristr($string,'&amp;view=companies') ) {

			// index.php?option=com_hotproperty&view=companies&layout=agents&id=2&Itemid=41
			
			$company_id = $this->extract('id',$string);

			// This condition allows listagent module to work so that $agent_id can be a javascript expression.
			if ( is_numeric($company_id) ) {
				$db->setQuery( "SELECT c.name AS company_name FROM #__hp_companies AS c" 
					.	"\nWHERE c.id = '".$company_id."'"
					.	"\nLIMIT 1"
				);
				$rows = $db->loadObjectList();
				$row = $rows[0];
			}
			
			$sefstring .= _HP_SEF_COMPANIES.'/';
			
			// Default virtual directory would be: _HP_SEF_COMPANIES/company_name/.
			// There is an alternative way to represent this as to allow List Companies module to work: _HP_SEF_COMPANIES/company_id
			if ( !is_numeric($company_id) || count($row) <= 0 ) {
				$sefstring .= $company_id.'/';
			} else {
				$sefstring .= sefencode($row->company_name).'/';
			}
			
			# View Company
			if ( stristr($string,"&amp;layout=agents") ) {

				// ordering and limits
				$sefstring .= $this->append_ordering_and_limits($string);
				
			# View Company Properties
			} else if ( stristr($string,"&amp;layout=properties") ) {
				
				$sefstring .= _HP_SEF_PROPERTIES.'/';
				
				// ordering and limits
				$sefstring .= $this->append_ordering_and_limits($string);
				
			# Send Enquiry Email
			} else if ( stristr($string,'&amp;layout=contact') ) {
				$sefstring .= _HP_SEF_CONTACT.'/';
				$referer = $this->extract('referer',$string);
				if ($referer) {
					$sefstring .= $this->append_query_string($sefstring,'referer',$referer);
				}
			}

		}

		# Search
		if (stristr($string,'&amp;view=searches')) {
		
			$sefstring .= _HP_SEF_SEARCHES.'/';

			// results
			if (stristr($string,'&amp;layout=results')) {
				$sefstring .= _HP_SEF_RESULTS.'/';
			}
			
			// ordering and limits
			$sefstring .= $this->append_ordering_and_limits($string);

		}

		return $sefstring;
	}

	/********************************************************
	* REVERT
	********************************************************/

	/**
	* Reverts to the Mambo query string out of the SEF advance URL
	* Input:
	*    $url_array, array, The SEF advance URL split in arrays (first custom virtual directory beginning at $pos+1)
	*    $pos, int, The position of the first virtual directory (component)
	* Output: $QUERY_STRING, string, Mambo query string (var1=$var1&var2=$var2)
	*    Note that this will be added to already defined first part (option=com_example&Itemid=$Itemid)
	**/
	function revert ($url_array, $pos) {

		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}

		$QUERY_STRING = '';

		if ( isset($url_array[$pos+2]) ) {
			switch($url_array[$pos+2]) {

				# Companies & Agents
				
				// _HP_SEF_COMPANIES/company_name/agent_name/

				case _HP_SEF_COMPANIES:

					# View Agent
					// _HP_SEF_COMPANIES/company_name/agent_name/

					if (isset($url_array[$pos+2]) && isset($url_array[$pos+3]) && isset($url_array[$pos+4]) && $url_array[$pos+4]!=_HP_SEF_PROPERTIES && $url_array[$pos+4]!=_HP_SEF_CONTACT && $url_array[$pos+2]!="" && (
							($url_array[$pos+3] == '' && $url_array[$pos+4] == '') || ($url_array[$pos+3] <> '' && $url_array[$pos+4] <> '')
						) ) {
						$agent_name = sefdecode($url_array[$pos+4]);
						$company_name = sefdecode($url_array[$pos+3]);
						
						$db->setQuery( "SELECT a.id FROM (#__hp_agents AS a, #__hp_companies AS c) "
							.	"\n WHERE c.id=a.company AND a.name ='".$agent_name."' AND c.name = '".$company_name."' LIMIT 1" );
						$agent_id = $db->loadResult();
						
						$_GET['view'] = $_REQUEST['view'] = 'agents';
						
						// _HP_SEF_COMPANIES/company_name/agent_name/contact/
						if ( isset($url_array[$pos+5]) && $url_array[$pos+5]==_HP_SEF_CONTACT ) {
							$layout = $_GET['layout'] = $_REQUEST['layout'] = 'contact';
						} else {
							$layout = $_GET['layout'] = $_REQUEST['layout'] = 'properties';
						}
						
						$_GET['id'] = $_REQUEST['id'] = $agent_id;
						$QUERY_STRING .= "&view=agents&layout=properties&id=$agent_id";
						
						// index.php?option=com_hotproperty&Itemid=41&view=agents&layout=properties&id=3

					} else {
					# View Company
						// _HP_SEF_COMPANIES/company_name/
						// _HP_SEF_COMPANIES/company_id/
						
						$_GET['view'] = $_REQUEST['view'] = 'companies';
						$QUERY_STRING .= '&view=companies';
						
						$layout = '';
						$company_id = '';
						
						if (isset($url_array[$pos+3]) && $url_array[$pos+3]!='') {
							$company_name = sefdecode($url_array[$pos+3]);
							$db->setQuery( "SELECT c.id FROM #__hp_companies AS c WHERE c.name ='".$company_name."' LIMIT 1" );
							$company_id = $db->loadResult();
							if ( $company_id == '' && is_numeric($company_name) )  {
								$company_id = $company_name;
							}
						}
						
						// _HP_SEF_COMPANIES/company_name/properties/
						if ( isset($url_array[$pos+4]) && $url_array[$pos+4]==_HP_SEF_PROPERTIES ) {
							$layout = 'properties';
						} else if ( isset($url_array[$pos+4]) && $url_array[$pos+4]==_HP_SEF_CONTACT ) {
							$layout = 'contact';
						} else if ($company_id!='') {
							$layout = 'agents';
						}
						
						if ($layout!='') {
							$_GET['layout'] = $_REQUEST['layout'] = $layout;
							$QUERY_STRING .= "&layout=$layout";
						}
						if ($company_id!='') {
							$_GET['id'] = $_REQUEST['id'] = $company_id;
							$QUERY_STRING .= "&id=$company_id";
						}
						
						// index.php?option=com_hotproperty&view=companies&layout=agents&id=2&Itemid=41
					}
					break;
				# View Agent
				// _HP_SEF_AGENTS/agent_id/
				case _HP_SEF_AGENTS:
					$_GET['view'] = $_REQUEST['view'] = 'agents';
					$QUERY_STRING .= "&view=agents&id=$agent_id";
					
					if (isset($url_array[$pos+3]) && $url_array[$pos+3]!='') {
						$agent_id = sefdecode($url_array[$pos+3]);
						
						$_GET['id'] = $_REQUEST['id'] = $agent_id;
						$QUERY_STRING .= "&id=$agent_id";
					}

					break;

				# View All Types
				case _HP_SEF_TYPES:

					$_GET['view'] = $_REQUEST['view'] = 'types';
					$QUERY_STRING .= "&view=types";

					// index.php?option=com_hotproperty&view=types
					break;
					
				# Send Enquiry Email
				case _HP_SEF_CONTACT:

					$company_id = $url_array[$pos+3];
					
					$_GET['view'] = $_REQUEST['view'] = 'companies';
					$_GET['layout'] = $_REQUEST['layout'] = 'contact';
					$_GET['id'] = $_REQUEST['id'] = $company_id;

					$QUERY_STRING .= "&view=companies&layout=contact&id=$company_id";
					
					// index.php?option=com_hotproperty&view=companies&layout=contact&id=2&referer=....

					break;
					
				# View Featured
				case _HP_SEF_FEATURED:
				
					$_GET['view'] = $_REQUEST['view'] = 'featured';
					$QUERY_STRING .= '&view=featured';

					break;

				case _HP_SEF_SEARCHES:
					# Standard Search

					$_GET['view'] = $_REQUEST['view'] = 'searches';
					$QUERY_STRING .= '&view=searches';

					if (isset($url_array[$pos+3]) && $url_array[$pos+3]==_HP_SEF_RESULTS) {
						$_GET['layout'] = $_REQUEST['layout'] = 'results';
						$QUERY_STRING .= '&layout=results';
					}
					break;
				
				case _HP_SEF_MANAGE:
					# Manage Property
					
					$_GET['view'] = $_REQUEST['view'] = 'properties';
					$_GET['layout'] = $_REQUEST['layout'] = 'manage';
					$QUERY_STRING .= '&view=properties&layout=manage';

					break;
					
				case _HP_SEF_PROPERTIES:
					$_GET['view'] = $_REQUEST['view'] = 'properties';
					$QUERY_STRING .= "&view=properties";
					break;
					
				# Types & Properties
				default:

					# View Property
					// /hotproperty/apartment/beach-apartments/
					if (isset($url_array[$pos+3]) && $url_array[$pos+3]!='') {
						$type_name = sefdecode($url_array[$pos+2]);
						$property_name = sefdecode($url_array[$pos+3]);
						
						$db->setQuery( "SELECT p.id FROM #__hp_properties AS p "
							.	"\nLEFT JOIN #__hp_prop_types AS t ON t.id = p.type"
							.	"\nWHERE p.name ='".$property_name."' AND t.name ='".$type_name."' LIMIT 1" 
							);
						$property_id = $db->loadResult();
						
						$_GET['view'] = $_REQUEST['task'] = 'properties';
						$_GET['layout'] = $_REQUEST['layout'] = 'property';
						$_GET['id'] = $_REQUEST['id'] = $property_id;

						$QUERY_STRING .= "&view=properties&layout=property&id=$property_id";
						// index.php?option=com_hotproperty&Itemid=41&view=properties&layout=property&id=56
						
					} else if (isset($url_array[$pos+2]) && $url_array[$pos+2]!='') {
					# View Type

						// /hotproperty/apartment/
						$type_name = sefdecode($url_array[$pos+2]);
						
						$db->setQuery( "SELECT t.id FROM #__hp_prop_types AS t WHERE t.name ='".$type_name."' LIMIT 1" );
						$type_id = $db->loadResult();
						
						$_GET['view'] = $_REQUEST['view'] = 'types';
						$_GET['layout'] = $_REQUEST['layout'] = 'properties';
						$_GET['id'] = $_REQUEST['id'] = $type_id;
						$QUERY_STRING .= "&view=types&layout=properties&id=$type_id";
						
						// index.php?option=com_hotproperty&Itemid=41&view=types&layout=properties&id=6

					} else {
						// /hotproperty/
						
						$_GET['view'] = $_REQUEST['view'] = 'home';
						$QUERY_STRING .= "&view=home";
						
						// index.php?option=com_hotproperty&view=home&Itemid=41
					}
					break;
			}
		}
		
		// Get the Itemid
		if (empty($_REQUEST['Itemid'])) {
			
			$view = isset($_GET['view']) ? $_GET['view'] : '';
			$layout = isset($_GET['layout']) ? $_GET['layout'] : '';
			$id = isset($_GET['id']) ? $_GET['id'] : '';
			
			$link = '';
			
			# Agent Contact Layout
			if ($view=='agents' && $layout=='contact') {
				$link = "&view=agents&layout=contact&id=$id";
				
			# Agent Layout
			} else if ($view=='agents' && $layout=='properties') {
				$link = "&view=agents&layout=properties&id=$id";

			# Agents Layout
			} else if ($view=='agents') {
				$link = '&view=agents';
				
			# Company Layout
			} else if ($view=='companies' && $layout=='agents') {
				$link = "&view=companies&layout=agents&id=$id";
				
			# Company Contact Layout
			} else if ($view=='companies' && $layout=='contact') {
				$link = "&view=companies&layout=contact&id=$id";
				
			# Company Properties Layout
			} else if ($view=='companies' && $layout=='properties') {
				$link = "&view=companies&layout=properties&id=$id";

			# Companies Layout
			} else if ($view=='companies') {
				$link = '&view=companies';

			# Featured Layout
			} else if ($view=='featured') {
				$link = '&view=featured';

			# Front Page Layout
			} else if ($view=='home') {
				$link = '&view=home';
				
			# Property Layout
			} else if ($view=='properties' && $layout=='property') {
				$link = "&view=properties&layout=property&id=$id";

			# Manage Layout
			} else if ($view=='properties' && $layout=='manage') {
				$link = '&view=properties&layout=manage';
				
			# Properties Layout
			} else if ($view=='properties') {
				$link = '&view=properties';
				
			# Search Results Layout
			} else if ($view=='searches' && $layout=='results') {
				$link = '&view=searches&layout=results';

			# Search Default Layout
			} else if ($view=='searches') {
				$link = '&view=searches';

			# Types Layout
			} else if ($view=='types') {
				$link = '&view=types';

			# Type's properties Layout
			} else if ($view=='types' && $layout=='properties') {
				$link = "&view=types&layout=properties&id=$id";
			
			}
			
			if ($link !='') {
				$query = "SELECT id FROM #__menu WHERE link='index.php?option=com_hotproperty$link' AND published=1 LIMIT 1";
			} else {
				# Default
				$query = "SELECT id FROM #__menu WHERE link LIKE 'index.php?option=com_hotproperty%' AND published=1 ORDER BY id LIMIT 1";
			}

			$db->setQuery( $query );
			$Itemid = $db->loadResult();
			
			if (!$Itemid) {
				# Default
				$query = "SELECT id FROM #__menu WHERE link LIKE 'index.php?option=com_hotproperty%' AND published=1 ORDER BY id LIMIT 1";
				$db->setQuery( $query );
				$Itemid = $db->loadResult();
			}
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$QUERY_STRING = $Itemid . $QUERY_STRING;
		}

		return $QUERY_STRING;
	}
	
	function append_ordering_and_limits($string) {
		global $sef_request_uri;
		$sefstring = '';
		
		// if query string already present in request save it
		if (stristr($sef_request_uri,'?')) {
			$temp = explode('?',$sef_request_uri);
			$query_string = $temp[count($temp)-1];
			$pairs = explode('&',$query_string);
			$query_array = array();
			foreach($pairs as $pair) {
				list($name,$value) = explode('=',$pair);
				if ($name=='Field') {
					$query_array[] = "$name=$value";
				}
			}
			if (count($query_array)>0) {
				$query_string = implode('&',$query_array);
				$sefstring .= '?'.$query_string;
			}
		}
		
		// ordering & ordering_dir
		$temp = explode("&amp;ordering=", $string);
		if (!empty($temp[1])) {
			$temp = explode("&", $temp[1]);
			$ordering = $temp[0];
			$sefstring .= $this->append_query_string($sefstring, 'ordering', $ordering);
		}
		
		$temp = explode("&amp;ordering_dir=", $string);
		if (!empty($temp[1])) {
			$temp = explode("&", $temp[1]);
			$ordering_dir = $temp[0];
			$sefstring .= $this->append_query_string($sefstring, 'ordering_dir', $ordering_dir);
		}
		
		// limit & limitstart
		$temp = explode('&amp;limit=', $string);
		if(!empty($temp[1])) {
			$temp = explode('&', $temp[1]);
			$limit = $temp[0];
			$sefstring .= $limit!='' ? $this->append_query_string($sefstring, 'limit', $limit) : '';
		}

		$temp = explode('&amp;limitstart=', $string);
		if(!empty($temp[1])) {
			$temp = explode('&', $temp[1]);
			$limitstart = $temp[0];
			$sefstring .= $limitstart!='' ? $this->append_query_string($sefstring, 'limitstart', $limitstart) : '';
		}
		
		return $sefstring;
	}
	
	function append_query_string($string, $name, $value) {
		if (stristr($string, '?')) {
			$append = '&';
		} else {
			$append = '?';
		}
		$append .= "$name=$value";
		return $append;
	}
	
	/*
	Extract variable from non-sef string
	*/
	function extract($name, $string) {
		$value = null;
		if (stristr($string,"&amp;$name=")) {
			$temp = explode("&amp;$name=", $string);
			$temp = explode('&', $temp[1]);
			$value = $temp[0];
		}
		return $value;
	}

}
?>